package com.diaoyun.zion.chinafrica.bis.impl;

import com.diaoyun.zion.chinafrica.bis.IItemSpider;
import com.diaoyun.zion.master.bo.TecentTranslateParam;
import com.diaoyun.zion.master.thread.TaskLimitSemaphore;
import com.diaoyun.zion.master.thread.TranslateCallable;
import com.diaoyun.zion.master.util.HttpClientUtil;
import com.diaoyun.zion.master.util.JsoupUtil;
import com.diaoyun.zion.master.util.ValidateUtils;
import net.sf.json.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.text.StringEscapeUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.methods.RequestBuilder;
import org.apache.http.client.protocol.HttpClientContext;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.message.BasicHeader;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

/**
 * 淘宝网数据爬虫
 */
@Component("tbItemSpider")
public class TbItemSpider implements IItemSpider {
    private static Logger logger = LoggerFactory.getLogger(TbItemSpider.class);

    //默认20条线程跑翻译
    private final TaskLimitSemaphore taskLimitSemaphore=new TaskLimitSemaphore(20);
    private final List<Map<String, Object>> futureList= new ArrayList<>();

    @Override
    public Map<String, Object> captureItem(String targetUrl) throws URISyntaxException, IOException, ExecutionException, InterruptedException {
        Long a=System.currentTimeMillis();
        //构建URI
        URI uri=new URIBuilder(targetUrl).build();
        //创建一个HttpContext对象，用来保存Cookie
        //HttpClientContext httpClientContext = HttpClientContext.create();
        //构造自定义的HttpClient对象
        CloseableHttpClient httpClient = HttpClientUtil.createBrowserClient();
        /* 构造请求对象 */
        HttpUriRequest httpUriRequest = RequestBuilder.get().setUri(uri).build();
        /* ****************获取商品详情链接******************* */
        //执行请求，传入HttpContext，将会得到请求结果的信息
        //httpClient.execute(httpUriRequest,httpClientContext)
        HttpResponse httpResponse=httpClient.execute(httpUriRequest);
        //拿到返回的HttpResponse的"实体"
        HttpEntity result = httpResponse.getEntity();
        String content = EntityUtils.toString(result);
        //获取商品详情url
        String usableSibUrl= JsoupUtil.getSibUrl(content);
        //解析商品sku信息
        Map<String,Object> propMap=JsoupUtil.getPropMap(content);
        //关闭连接
        httpClient.close();
        Long b=System.currentTimeMillis();
        logger.info("获取淘宝数据，第一段时间(毫秒)："+(b-a));
        //调用腾讯ai，翻译规格
        translateProp(propMap);
        //等待翻译结果
        for(Map<String,Object> futureMap:futureList) {
            Future<Map<String,Object>> future= (Future<Map<String, Object>>) futureMap.get("future");
            Map<String,Object> valeMap = (Map<String,Object> ) futureMap.get("value");
            while(!future.isDone());//Future返回如果没有完成，则一直循环等待，直到Future返回完成
            Map<String,Object> resultMap=future.get();
            String targetText="unknow";
            if(resultMap!=null&&(int)resultMap.get("ret")==0) {
                Map<String,Object> dataMap=(Map<String,Object>)resultMap.get("data");
                targetText= (String) dataMap.get("target_text");
                valeMap.put("translate",targetText);
            } else {
                logger.error("翻译出错");
            }
        }
        //关闭线程池 不能关闭，否则下次调用不了
        //taskLimitSemaphore.shutdown();
        Long c=System.currentTimeMillis();
        logger.info("翻译时间(毫秒)："+(c-b));

        /* ****************获取商品详情******************* */
        //删除需要登录的模块
        usableSibUrl=deleteLoginModule(usableSibUrl);
        URI sibUri=new URIBuilder(usableSibUrl).build();
        //构造请求对象
        HttpUriRequest sibUriRequest = RequestBuilder.get().setUri(sibUri).build();
        BasicHeader basicHeader=new BasicHeader("referer", targetUrl);
        //构造自定义的HttpClient对象
        CloseableHttpClient sibClient = HttpClientUtil.createBrowserClient(basicHeader);
        //创建一个HttpContext对象，用来保存Cookie
        //HttpClientContext sibClientContext = HttpClientContext.create();
        //执行请求，传入HttpContext，将会得到请求结果的信息
        HttpResponse sibResponse=sibClient.execute(sibUriRequest);
        //拿到返回的HttpResponse的"实体"
        HttpEntity sibResult = sibResponse.getEntity();
        String sibContent = EntityUtils.toString(sibResult);
        //unicode 解码
        sibContent= StringEscapeUtils.unescapeJava(sibContent);
        Map sibMap= JSONObject.fromObject(sibContent);
        sibMap.put("Jprop",propMap);
        logger.info(sibMap.toString());
        //从请求结果中获取Cookie，此时的Cookie已经带有登录信息了
        //CookieStore store = httpClientContext.getCookieStore();
        sibClient.close();
        Long d=System.currentTimeMillis();
        logger.info("获取详情时间(毫秒)："+(d-c));
        logger.info("爬取数据总共耗费时间(毫秒)："+(d-a));
        return sibMap;
    }



    /**
     * 翻译规格属性
     * @param propMap 规格属性MAP
     * @throws ExecutionException
     * @throws InterruptedException
     */
    private void translateProp(Map<String, Object> propMap) throws ExecutionException, InterruptedException {
        /*腾讯翻译*/
        for(Map.Entry<String,Object>entry : propMap.entrySet()) {
            String key=entry.getKey();
            Map <String,Object> value= (Map<String, Object>) entry.getValue();
            //翻译属性名
            if(ValidateUtils.isChinese(key)) {
                translateText(value,key);
            }
            //翻译sku title
            if(value!=null&&value.size()>0) {
                translateTitle(value);
            }
        }
    }

    /**
     * 翻译规格说明
     * @param skuMap
     * @throws ExecutionException
     * @throws InterruptedException
     */
    private void translateTitle(Map<String, Object> skuMap) throws ExecutionException, InterruptedException {
        for(Map.Entry<String,Object>entry : skuMap.entrySet()) {
            String key=entry.getKey();
            if(entry.getValue() instanceof Map) {
                Map<String, Object> value = (Map<String, Object>) entry.getValue();
                String title = (String) value.get("title");
                if (StringUtils.isNotBlank(title)) {
                    //翻译属性名
                    if (ValidateUtils.isContainChinese(title)) {
                        translateText(value,title);
                        //value.put("translate", translate);
                    }
                }
            }
        }
    }

    /**
     * 翻译文本
     * @param valeMap
     * @param text
     * @throws ExecutionException
     * @throws InterruptedException
     */
    private void translateText(Map<String,Object> valeMap,String text) throws ExecutionException, InterruptedException {
        TecentTranslateParam tecentTranslateParam=new TecentTranslateParam(text);
        Future<Map<String,Object>> future = null;
        try {
            future = taskLimitSemaphore.submit(new TranslateCallable(tecentTranslateParam));
            Map<String,Object> map=new HashMap<>();
            map.put("future",future);
            map.put("value",valeMap);
            futureList.add(map);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 去除需要登录或者不需要返回的参数
     * @param usableSibUrl
     * @return
     */
    private String deleteLoginModule(String usableSibUrl) {
        usableSibUrl=usableSibUrl.replaceAll("couponActivity,","");
        usableSibUrl=usableSibUrl.replaceAll("soldQuantity,","");
        usableSibUrl=usableSibUrl.replaceAll("tradeContract,","");
        usableSibUrl=usableSibUrl.replaceAll("upp,","");
        // TODO 运费格式有问题，暂去除
        usableSibUrl=usableSibUrl.replaceAll("deliveryFee,","");
        usableSibUrl=usableSibUrl.replaceAll("delivery,","");
        return usableSibUrl;
    }

}
