package com.diaoyun.zion.chinafrica.controller;

import com.diaoyun.zion.chinafrica.constant.KeyConstant;
import com.diaoyun.zion.chinafrica.entity.TbCfUserInfoEntity;
import com.diaoyun.zion.chinafrica.service.TbCfUserInfoService;
import com.diaoyun.zion.chinafrica.vo.TbCfUserInfoVo;
import com.diaoyun.zion.master.base.BaseController;
import com.diaoyun.zion.master.base.Result;
import com.diaoyun.zion.master.base.ResultCode;
import com.diaoyun.zion.master.util.PasswordProvider;
import com.diaoyun.zion.master.validator.Validator;
import com.diaoyun.zion.master.validator.annotation.ValidateParam;
import freemarker.template.TemplateException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.mail.EmailException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;

/**
 * 用户相关api
 *
 * @author G
 */
@Api(tags = "用户相关")
@RestController
@RequestMapping("/user")
public class TbCfUserInfoController extends BaseController {

    private static Logger logger = LoggerFactory.getLogger(TbCfUserInfoController.class);

    @Autowired
    private HttpServletRequest request; //自动注入request

    @Autowired
    private TbCfUserInfoService tbCfUserInfoService;

    /**
     * 获取邮箱验证码
     */
    @ApiOperation(value = "获取邮箱验证码")
    @GetMapping(value = "/register/identifyCode/{nick}/{email}")
    @ResponseBody
    public Result getUserIdentifyCode(@ApiParam @PathVariable("email") @ValidateParam({Validator.NOT_BLANK}) String email,
                                      @ApiParam @PathVariable("nick") @ValidateParam({Validator.NOT_BLANK}) String nick) throws EmailException, TemplateException, IOException {
        Integer identifyCode = tbCfUserInfoService.sendRegisterIdentifyCode(email, nick);
        request.getSession().setAttribute(KeyConstant.IDENTIFY_CODE, identifyCode);
        return new Result(identifyCode, "验证码");
    }

    /**
     * 获取存在session中的验证码
     */
    /*@ApiOperation(value = "获取存在session中的验证码")
    @GetMapping(value ="/register/identifyCode/old")
    public String getDeliveryList() {
        String identifyCode= (int) request.getSession().getAttribute(KeyConstant.IDENTIFY_CODE);
        return identifyCode;
    }
    */

    @ApiOperation("修改用户信息")
    @PutMapping(value="/{userId}")
    @ResponseBody
    public Result updateUserInfo(@ApiParam @PathVariable("userId")String userId,@RequestBody TbCfUserInfoVo tbCfUserInfoVo) {
        TbCfUserInfoEntity tbCfUserInfoEntity = new TbCfUserInfoEntity();
        BeanUtils.copyProperties(tbCfUserInfoEntity,tbCfUserInfoVo);
        int res=tbCfUserInfoService.update(tbCfUserInfoEntity);
        return new Result(res, "修改成功");
    }

    @ApiOperation("修改用户密码")
    @PutMapping(value="/password/{userId}/{newPassword}/{oldPassword}")
    @ResponseBody
    public Result updatePassWord(@ApiParam @PathVariable @ValidateParam({Validator.NOT_BLANK}) String userId,
                                 @ApiParam @PathVariable @ValidateParam({Validator.NOT_BLANK}) String newPassword
            ,@ApiParam @PathVariable @ValidateParam({Validator.NOT_BLANK}) String oldPassword) {
        Result result =tbCfUserInfoService.updatePassWord(userId,PasswordProvider.encrypt(newPassword),PasswordProvider.encrypt(oldPassword));
        int code=result.getCode();
        if(code==ResultCode.SUCCESS) {
            tbCfUserInfoService.logout(request);
        }
        return result;
    }



}
