package com.diaoyun.zion.chinafrica.service.impl;

import com.diaoyun.zion.chinafrica.client.SessionUtils;
import com.diaoyun.zion.chinafrica.constant.EmailTemplateConstant;
import com.diaoyun.zion.chinafrica.dao.TbCfUserInfoDao;
import com.diaoyun.zion.chinafrica.entity.TbCfUserInfoEntity;
import com.diaoyun.zion.chinafrica.service.TbCfUserInfoService;
import com.diaoyun.zion.chinafrica.vo.TbCfUserInfoVo;
import com.diaoyun.zion.master.base.Result;
import com.diaoyun.zion.master.base.ResultCode;
import com.diaoyun.zion.master.base.StateConstant;
import com.diaoyun.zion.master.bo.EmailTemplateBo;
import com.diaoyun.zion.master.common.TokenManager;
import com.diaoyun.zion.master.config.DomainProperties;
import com.diaoyun.zion.master.enums.SexEnum;
import com.diaoyun.zion.master.enums.TrueFalseEnum;
import com.diaoyun.zion.master.exception.ValidateException;
import com.diaoyun.zion.master.util.*;
import com.diaoyun.zion.master.validator.Validator;
import freemarker.template.TemplateException;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.mail.EmailException;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 用户表Service实现类
 *
 * @author lipengjun
 * @date 2019-08-14 09:11:47
 */
@Service("tbCfUserInfoService")
public class TbCfUserInfoServiceImpl implements TbCfUserInfoService {
    @Autowired
    private TbCfUserInfoDao tbCfUserInfoDao;

    @Resource(name="redisTokenManager")
    private TokenManager tokenManager;

    @Autowired
    private EmailHelper emailHelper;

    @Autowired
    private DomainProperties domainProperties;

    @Override
    public TbCfUserInfoEntity queryObject(String userId) {
        return tbCfUserInfoDao.queryObject(userId);
    }

    @Override
    public List<TbCfUserInfoEntity> queryList(Map<String, Object> map) {
        return tbCfUserInfoDao.queryList(map);
    }

    @Override
    public int queryTotal(Map<String, Object> map) {
        return tbCfUserInfoDao.queryTotal(map);
    }

    @Override
    public int save(TbCfUserInfoEntity tbCfUserInfo) {
        tbCfUserInfo.setUserId(IdUtil.createIdbyUUID());
        return tbCfUserInfoDao.save(tbCfUserInfo);
    }

    @Override
    public int update(TbCfUserInfoEntity tbCfUserInfo) {
        return tbCfUserInfoDao.update(tbCfUserInfo);
    }

    @Override
    public int delete(String userId) {
        return tbCfUserInfoDao.delete(userId);
    }

    @Override
    public int deleteBatch(String[] userIds) {
        return tbCfUserInfoDao.deleteBatch(userIds);
    }

    @Override
    public Integer sendRegisterIdentifyCode(String email,String nick) throws EmailException, TemplateException, IOException {
        EmailTemplateBo emailTemplateBo = new EmailTemplateBo();
        Integer randomCode = RandomCodeHelper.producedRandomCode(6);
        emailTemplateBo.setNick(nick);
        emailTemplateBo.setIdentifyCode(randomCode);
        Integer identifyCode= emailHelper.sendIdentifyEmail(email, EmailTemplateConstant.REGISTER,emailTemplateBo);
        return identifyCode;
    }

    @Override
    public TbCfUserInfoVo registerAndLogin(TbCfUserInfoVo tbCfUserInfoVo) {
        Validator.NOT_BLANK.validate("邮箱",tbCfUserInfoVo.getEmail());
        TbCfUserInfoEntity existUser=findByAccount(tbCfUserInfoVo.getEmail());
        if(existUser==null) {
            TbCfUserInfoEntity tbCfUserInfoEntity =new TbCfUserInfoEntity();
            /**
             * 填充必要信息
             */
            fillUserNecessayInfo(tbCfUserInfoVo);
            //加密密码
            String password= PasswordProvider.encrypt(tbCfUserInfoVo.getPassword());
            tbCfUserInfoVo.setPassword(password);
            BeanUtils.copyProperties(tbCfUserInfoVo,tbCfUserInfoEntity);
            tbCfUserInfoDao.save(tbCfUserInfoEntity);
        } else {
            throw new ValidateException("邮箱已被注册！");
        }

        return tbCfUserInfoVo;
    }

    @Override
    public Result login(String ip, String account, String password) {
        Result result = new Result();
        TbCfUserInfoEntity user = findByAccount(account);
        if (user == null) {
            result.setCode(ResultCode.ERROR).setMessage("登录名不存在");
        }
        else if (!user.getPassword().equals(password)) {
            result.setCode(ResultCode.ERROR).setMessage("密码不正确");
        }
        else if (TrueFalseEnum.FALSE.getValue().equals(user.getEnableFlag())) {
            result.setCode(ResultCode.ERROR).setMessage("已被用户禁用");
        }
        else {
            user.setLastLoginIp(ip);
            user.setLoginCount(user.getLoginCount() + 1);
            user.setLastLoginTime(new Date());
            tbCfUserInfoDao.update(user);
            result.setData(user);
        }
        return result;
    }

    @Override
    public TbCfUserInfoEntity findByAccount(String account) {
        TbCfUserInfoEntity tbCfUserInfoEntity=tbCfUserInfoDao.findByAccount(account);
        return tbCfUserInfoEntity;
    }

    @Override
    public Result updatePassWord(String userId, String newPassword, String oldPassword) {
        Result result= new Result();
        int res=tbCfUserInfoDao.updatePassWord(userId,newPassword,oldPassword);
        if(res>0) {
            result.setMessage("修改成功");
        } else {
            result.setCode(ResultCode.ERROR).setMessage("旧密码错误");
        }
        return result;
    }

    @Override
    public Result logout(HttpServletRequest request) {
        String token = CookieUtils.getCookie(request, TokenManager.TOKEN);
        if (StringUtils.isNotBlank(token)) {
            tokenManager.remove(token);
        }
        SessionUtils.invalidate(request);
        return new Result("已退出登录");
    }


    private void fillUserNecessayInfo(TbCfUserInfoVo tbCfUserInfoVo) {
        tbCfUserInfoVo.setUserId(IdUtil.createIdbyUUID());
        tbCfUserInfoVo.setUserNo(IdUtil.createIdByDate());
        tbCfUserInfoVo.setAvatar(domainProperties.getProperty("user.avatar"));
        tbCfUserInfoVo.setPhoneFlag(StateConstant.INVALID);
        tbCfUserInfoVo.setLoginCount(0);
        tbCfUserInfoVo.setCreateTime(new Date());
        tbCfUserInfoVo.setSex(SexEnum.UNKNOW.getCode());
        tbCfUserInfoVo.setInvitedCount(0);
        tbCfUserInfoVo.setEnableFlag(StateConstant.VALID);
    }


}
