package com.diaoyun.zion.master.thread;

import com.diaoyun.zion.master.bo.TecentTranslateParam;
import com.diaoyun.zion.master.config.DomainProperties;
import com.diaoyun.zion.master.enums.TranslateLanguageEnum;
import com.diaoyun.zion.master.util.HttpClientUtil;
import com.diaoyun.zion.master.util.IdUtil;
import com.diaoyun.zion.master.util.SortUtil;
import com.diaoyun.zion.master.util.SpringContextUtil;
import net.sf.json.JSONObject;
import org.apache.commons.beanutils.BeanUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.concurrent.Callable;

/*腾讯翻译类
 * https://api.ai.qq.com/fcgi-bin/nlp/nlp_texttranslate
 * */
public class TranslateCallable implements Callable<Map<String, Object>> {

    private static Logger logger = LoggerFactory.getLogger(TranslateCallable.class);
    //申请的app_id
    private Integer app_id;
    //申请的app_url
    private String app_url;

    private TecentTranslateParam param;


    public TranslateCallable(TecentTranslateParam param) {
        DomainProperties domainProperties = (DomainProperties) SpringContextUtil.getBean("domainProperties");
        this.app_id = Integer.valueOf(domainProperties.getProperty("tencent.translate.app_id"));
        this.app_url = domainProperties.getProperty("tencent.translate.app_url");
        this.param = param;
    }

    @Override
    public Map<String, Object> call() throws Exception {
        //logger.info("call()方法被自动调用！！！    " + Thread.currentThread().getName());
        //填充参数
        fillTranslateParam(param);
        Map<String, Object> paramMap = BeanUtils.describe(param);
        // TODO 去除class说明，或许可以尝试别的方法
        paramMap.remove("class");
        SortedMap<String, Object> sortMap = new TreeMap<String, Object>(paramMap);
        String sign = SortUtil.createSign(sortMap);
        paramMap.put("sign", sign);
        //发送请求
        String resultStr = HttpClientUtil.createConnection(this.app_url, paramMap);
        Map<String, Object> resultMap = JSONObject.fromObject(resultStr);
        return resultMap;
    }

    /**
     * 填充翻译时参数
     *
     * @param param
     */
    private void fillTranslateParam(TecentTranslateParam param) {
        if (param.getApp_id() == null) {
            param.setApp_id(this.app_id);
        }
        if (param.getNonce_str() == null) {
            String randomLetters = IdUtil.createIdbyUUID();
            param.setNonce_str(randomLetters);
        }
        if (param.getTime_stamp() == null) {
            long currentTime = System.currentTimeMillis() / 1000;
            param.setTime_stamp(currentTime);
        }
        if (param.getSource() == null) {
            param.setSource(TranslateLanguageEnum.ZH.getValue());
        }
        if (param.getTarget() == null) {
            param.setTarget(TranslateLanguageEnum.EN.getValue());
        }
    }
}
